class Course:

    # all of the labels for the data of a course
    labels = ['Course code', 'Course name', 'Course description',
             'Course exclusion', 'Course prerequisite',
             'Course corequisite', 'Course preparation',
             'Course distribution requirement',
             'Course breadth requirement']

    def __init__(self, course_code, course_name, course_description,
                 exclusion, prerequisite, corequisite, recommended,
                 distribution_requirement, breadth_requirement):
        """
        Represents a single Course

        :param course_code: The code of this Course
        :type course_code: str
        :param course_name: The name of this Course
        :type course_name: str
        :param course_description: The description of this Course
        :type course_description: str
        :param exclusion: The exclusions of this Course
        :type exclusion: str
        :param prerequisite: The prerequisites of this Course
        :type prerequisite: str
        :param corequisite: The corequities of this Course
        :type corequisite: str
        :param recommended: The recommended preparation of this Course
        :type recommended: str
        :param distribution_requirement: The distribution requirement of this
                                         Course
        :type distribution_requirement: str
        :param breadth_requirement: The breadth requirement of this Course
        :type breadth_requirement: str
        """
        self.course_code = Course._format(course_code)
        self.course_name = Course._format(course_name)
        self.course_description = Course._format(course_description)
        self.exclusion = Course._format(exclusion)
        self.prerequisite = Course._format(prerequisite)
        self.corequisite = Course._format(corequisite)
        self.recommended = Course._format(recommended)
        self.distribution_requirement = Course._format(distribution_requirement)
        self.breadth_requirement = Course._format(breadth_requirement)

        self.data = [self.course_code, self.course_name,
                     self.course_description, self.exclusion,
                     self.prerequisite, self.corequisite, self.recommended,
                     self.distribution_requirement, self.breadth_requirement]

    def __str__(self):
        """
        Returns a string representation of this course

        :return: A string representation of this Course
        :rtype: str
        """
        delimiter = '\n' * 2
        return delimiter.join([label + ': ' + data for label, data in \
                zip(Course.labels, self.data)])

    def to_data(self):
        """
        Return the information of a course as a set of key-value pairs in a
        dictionary.

        :return: A dictionary containing the course information as its key-value
                 pairs
        :rtype: dict{str, str}
        """
        course = {}
        for label, data in zip(Course.labels, self.data):
            course[label] = data
        return course

    @staticmethod
    def _format(val):
        """
        Used to format the values of the information and removes leading and
        trailing white space characters.

        If the value is an empty string or contains only white space characters,
        return 'None'

        :param val: The value to be formatted
        :type val: str
        :return: The formatted value
        :rtype: str
        """
        val = val.strip()
        return val if val is not "" else 'None'



import re

from raw_html import get_html


class FacultyCalender:
    """
    Represents an instance of a particular faculty calender for a certain year.
    """

    def __init__(self, faculty, session, url):
        """
        Initialize a new Faculty Calender for the given url

        :param faculty: The faculty the calender belongs to
        :type faculty: Faculty
        :param session: The session of the calender
        :type session: Session
        :param url: The url to the specified year's calender
        :type url: URL
        """
        self.faculty = faculty
        self.session = session
        self.url = url
        self.department_calenders = []

    def get_department_calenders(self):
        """
        Return a list of Department Calender from this year of this Faculty
        """
        if self.department_calenders:
            return self.department_calenders
        else:
            return self._generate_department_calenders_from_html()

    def _generate_department_calenders_from_html(self):
        """
        Using the unescaped raw html of the calender, create and return all the
        department calenders found.
        """
        raise NotImplementedError



import json


from faculty_calender import FacultyCalender
from faculty_calender_resources.faculty_of_arts_and_science_calender import FacultyOfArtsAndScienceCalender
from faculty_calender_resources.session import Session
from faculty_calender_resources.session import Season
from faculty_calender_resources.url import URL

cal = FacultyOfArtsAndScienceCalender(Session(2016, Season.FALL),
                                      URL("http://calendar.artsci.utoronto.ca"))

print('getting department calenders')
deps = cal.get_department_calenders()

course_datas = []

print('getting course datas')
for dep in deps:
    for course in dep.get_courses():
        course_datas.append(course.to_data())

# print(type(course_datas))
# print(type(course_datas[0]))
# print(type(course_datas[0]['Course code']))

file = open("courses.json", 'w')
file.write(json.dumps({"courses":course_datas}, indent=4))



import re

from faculty_calender_resources.department import Department
from course import Course

class DepartmentCalender:

    def __init__(self, session, url):
        """
        Initialize a new Department Calender for the given url

        :param session: The session of the calender
        :type session: Session
        :param url: The url to the specified year's calender
        :type url: URL
        """
        self.session = session
        self.url = url
        self.department = self._find_department()
        self.courses = []

    # regex used for the _find_department method
    _department_name = re.compile(r"<h1>(.*)<\/h1>")

    def _find_department(self):
        """
        Return the Department found at the given url

        :return: The Department
        :rtype: Department
        """
        matches = DepartmentCalender._department_name.\
                findall(self.url.raw_html)

        # only a single h1 tag in the html, and it is the department name
        return Department(matches[0])

    # please don't touch this regular expression without fully understanding it
    # it has been adjusted after many iterations after finding strange
    # formatting in the raw html, so any changes is not advised

    # regular expression used to filter out the course data
    regex = '<a name="([A-Z]{3}\\d\\d\\d[A-Z]\\d)"><\/a>'\
            '<span class="strong">\\1\\s*(.*?)<\/span>\\s*'\
            '<\/?p(.*?)?>([\\s\\S]*?)<\/?p>\\s*'\
            '(Exclusion:\\s*(.*?)|Prerequisite:\\s*(.*?)|'\
            'Corequisite:\\s*(.*?)|Recommended Preparation:\\s*(.*?))?(<br>)?'\
            '(Exclusion:\\s*(.*?)|Prerequisite:\\s*(.*?)|'\
            'Corequisite:\\s*(.*?)|Recommended Preparation:\\s*(.*?))?(<br>)?'\
            '(Exclusion:\\s*(.*?)|Prerequisite:\\s*(.*?)|'\
            'Corequisite:\\s*(.*?)|Recommended Preparation:\\s*(.*?))?(<br>)?'\
            '(Exclusion:\\s*(.*?)|Prerequisite:\\s*(.*?)|'\
            'Corequisite:\\s*(.*?)|Recommended Preparation:\\s*(.*?))?(<br>)?'\
            '(Distribution Requirement Status:\\s*(.*?)\\s*)?(<br>)?'\
            '(Breadth Requirement:\\s*(.*?)\\s*)?<br>'

    _course = re.compile(regex, re.DOTALL)

    def get_courses(self):
        """
        Returns a list of all the courses in this Department Calender.

        :return: list of all courses in this DepartmentCalender
        :rtype: list[Course]
        """
        # if the list has been generated
        if self.courses:
            return self.courses

        # generate list if necessary
        courses_data = DepartmentCalender._course.findall(self.url.raw_html)

        for course_data in courses_data:
            self.courses.append(DepartmentCalender._create_course(course_data))

        return self.courses.copy()

    @staticmethod
    def _create_course(data):
        """
        Create a course object from the data extracted using the above regex

        :param data: The data extracted using the above regex
        :type data: tuple(str, ...)
        :return: A course object
        :rtype: Course
        """
        # these numbers come from the group numbers from the regex above
        # '_course' count them if you wanna
        course_code = DepartmentCalender._erase_html(data[0])
        course_name = DepartmentCalender._erase_html(data[1])
        course_description = DepartmentCalender._erase_html(data[3])
        exclusion = DepartmentCalender._erase_html(
            data[5] + data[11] + data[17] + data[23])
        prerequisite = DepartmentCalender._erase_html(
            data[6] + data[12] + data[18] + data[24])
        corequisite = DepartmentCalender._erase_html(
            data[7] + data[13] + data[19] + data[25])
        recommended = DepartmentCalender._erase_html(
            data[8] + data[14] + data[20] + data[26])
        distribution_requirement = DepartmentCalender._erase_html(
            data[29])
        breath_requirement = DepartmentCalender._erase_html(data[32])

        print("found course: {}".format(course_code))

        return Course(course_code, course_name, course_description,
                      exclusion, prerequisite, corequisite, recommended,
                      distribution_requirement, breath_requirement)

    _tags = re.compile('<.*?>', re.DOTALL)

    @staticmethod
    def _erase_html(data):
        """
        Erases any remaining html tags in the text.

        :param data: The raw data
        :type data: str
        :return: The data after removing remaining html tags
        :rtype: str
        """
        return DepartmentCalender._tags.sub('', data)



from distutils.core import setup
setup(
  name = 'courspider',
  packages = ['courspider'],
  version = '0.1',
  description = 'A webscraper API to scrape course data from u of t calenders',
  author = 'Tony Xiao',
  author_email = 'zylphrex@gmail.com',
  url = 'https://github.com/Zylphrex/courspider',
  download_url = 'https://github.com/Zylphrex/courspider/tarball/0.1',
  keywords = ['u of t', 'University', 'Toronto', 'course', 'data'],
  classifiers = [],
)
